import torch
import torchvision
from torch.autograd import Variable
import torch.utils.data.dataloader as Data
from torch.utils.tensorboard import SummaryWriter
import os

# The following switch allows the program runs locally and in the Agit environment without modifications.
if 'CLOUD_PROVIDER' in os.environ and os.environ['CLOUD_PROVIDER'] == 'Agit':
    logdir = '/root/.agit'
else:
    logdir = './runs'
    
writer = SummaryWriter(logdir)

train_data = torchvision.datasets.MNIST(
    './mnist', train=True, transform=torchvision.transforms.ToTensor(), download=True
)
test_data = torchvision.datasets.MNIST(
    './mnist', train=False, transform=torchvision.transforms.ToTensor()
)

print(f"train_data: {train_data.data.size()}")
print(f"train_labels: {train_data.targets.size()}")
print(f"test_data: {test_data.data.size()}")

train_loader = Data.DataLoader(dataset=train_data, batch_size=64, shuffle=True)
test_loader = Data.DataLoader(dataset=test_data, batch_size=64)

device=torch.device("cuda:0" if torch.cuda.is_available() else "cpu")

class ConvNet(torch.nn.Module):
    def __init__(self):
        super(ConvNet, self).__init__()
        self.conv1 = torch.nn.Sequential(
            torch.nn.Conv2d(1, 32, 3, 1, 1),
            torch.nn.ReLU(),
            torch.nn.MaxPool2d(2))
        self.conv2 = torch.nn.Sequential(
            torch.nn.Conv2d(32, 64, 3, 1, 1),
            torch.nn.ReLU(),
            torch.nn.MaxPool2d(2)
        )
        self.conv3 = torch.nn.Sequential(
            torch.nn.Conv2d(64, 64, 3, 1, 1),
            torch.nn.ReLU(),
            torch.nn.MaxPool2d(2)
        )
        self.dense = torch.nn.Sequential(
            torch.nn.Linear(64 * 3 * 3, 128),
            torch.nn.ReLU(),
            torch.nn.Linear(128, 10)
        )

    def forward(self, x):
        conv1_out = self.conv1(x)
        conv2_out = self.conv2(conv1_out)
        conv3_out = self.conv3(conv2_out)
        res = conv3_out.view(conv3_out.size(0), -1)
        out = self.dense(res)
        return out

model = ConvNet().to(device)
print(model)

optimizer = torch.optim.Adam(model.parameters())
loss_func = torch.nn.CrossEntropyLoss()

for epoch in range(10):
    print(f"epoch: {epoch + 1}")
    # training
    train_loss = 0.
    train_acc = 0.
    for data, target in train_loader:
        data, target = Variable(data).to(device), Variable(target).to(device)
        out = model(data)
        loss = loss_func(out, target)
        train_loss += loss.item()
        pred = torch.max(out, 1)[1]
        train_correct = (pred == target).sum()
        train_acc += train_correct.item()
        optimizer.zero_grad()
        loss.backward()
        optimizer.step()
    
    training_loss = round(train_loss/len(train_data), 4)
    training_acc = round(train_acc/len(train_data), 4)
    writer.add_scalar("loss/train", training_loss, epoch)
    writer.add_scalar("acc/train", training_acc, epoch)
        
    print(f"training loss: {training_loss}, acc: {training_acc}")

    # evaluation
    with torch.no_grad():
        model.eval()
        eval_loss = 0.
        eval_acc = 0.
        for data, target in test_loader:
            data, target = data.to(device), target.to(device)
            out = model(data)
            loss = loss_func(out, target)
            eval_loss += loss.item()
            pred = torch.max(out, 1)[1]
            eval_correct = (pred == target).sum()
            eval_acc += eval_correct.item()
    
    eval_loss = round(train_loss/len(train_data), 4)
    eval_acc = round(train_acc/len(train_data), 4)
    writer.add_scalar("loss/eval", eval_loss, epoch)
    writer.add_scalar("acc/eval", eval_acc, epoch)
    
    print(f"eval loss: {eval_loss}, acc: {eval_acc}")
    
save_dir = "/root/.agit/"
torch.save(model, os.path.join(save_dir, "pytorch_mnist_model.pkl"))

writer.close()